@@ -426,11 +426,23 @@ func TestCommandLine(t *testing.T) {
 		{[]string{`BEGIN { print "1"; print "2"|"cat" }`}, "", "1\n2\n", ""},
 		{[]string{`BEGIN { print "1"; "echo 2" | getline x; print x }`}, "", "1\n2\n", ""},
 
+		// The "nextfile" statement
+		{[]string{`{ print FILENAME ":" FNR "/" NR ": " $0 }  FNR==2 { nextfile }`,
+			"testdata/g.5", "-", "testdata/g.6"},
+			"a\nb\nc\n", `
+testdata/g.5:1/1: one
+testdata/g.5:2/2: two 2
+-:1/3: a
+-:2/4: b
+testdata/g.6:1/5: Uno
+testdata/g.6:2/6: Duo
+`[1:], ""},
+
 		// Parse error formatting
 		{[]string{"`"}, "", "", "<cmdline>:1:1: unexpected char\n`\n^"},
 		{[]string{"BEGIN {\n\tx*;\n}"}, "", "", "<cmdline>:2:4: expected expression instead of ;\n    x*;\n      ^"},
 		{[]string{"BEGIN {\n\tx*\r\n}"}, "", "", "<cmdline>:2:4: expected expression instead of <newline>\n    x*\n      ^"},
-		{[]string{"-f", "-"}, "\n ++", "", "<stdin>:2:4: expected expression instead of <newline>\n ++\n   ^"},
+		{[]string{"-f", "-"}, "\n $", "", "<stdin>:2:3: expected expression instead of <newline>\n $\n  ^"},
 		{[]string{"-f", "testdata/parseerror/good.awk", "-f", "testdata/parseerror/bad.awk"},
 			"", "", "testdata/parseerror/bad.awk:2:3: expected expression instead of <newline>\nx*\n  ^"},
 		{[]string{"-f", "testdata/parseerror/bad.awk", "-f", "testdata/parseerror/good.awk"},
@@ -467,13 +479,19 @@ func runGoAWK(args []string, stdin string) (stdout, stderr string, err error) {
 }
 
 func runAWKs(t *testing.T, testArgs []string, testStdin, testOutput, testError string) {
+	t.Helper()
+
 	t.Run("awk", func(t *testing.T) {
+		t.Helper()
 		var args []string
 		if strings.Contains(awkExe, "gawk") {
 			args = append(args, "--posix")
 		}
 		args = append(args, testArgs...)
 		cmd := exec.Command(awkExe, testArgs...)
+		if runtime.GOOS != "windows" {
+			cmd.Env = []string{"LC_ALL=en_US.UTF-8"}
+		}
 		if testStdin != "" {
 			cmd.Stdin = strings.NewReader(testStdin)
 		}
@@ -496,6 +514,7 @@ func runAWKs(t *testing.T, testArgs []string, testStdin, testOutput, testError s
 	})
 
 	t.Run("goawk", func(t *testing.T) {
+		t.Helper()
 		stdout, stderr, err := runGoAWK(testArgs, testStdin)
 		if err != nil {
 			stderr = strings.TrimSpace(stderr)
@@ -609,7 +628,7 @@ func TestGoAWKSpecificOptions(t *testing.T) {
 		// Debug options (don't test -dt as its output is not stable)
 		{[]string{"-d", `$1 { print 1+1 }`}, "", `
 $1 {
-    print (1 + 1)
+    print 1 + 1
 }
 `[1:], ""},
 		{[]string{"-da", `$1 { print 1+1 }`}, "", `
@@ -625,7 +644,7 @@ $1 {
 `[1:], ""},
 		{[]string{"-d", "-da", `$1 { print 1+1 }`}, "", `
 $1 {
-    print (1 + 1)
+    print 1 + 1
 }
         // pattern
 0000    FieldInt 1
