@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"math"
 	"regexp"
+	"strconv"
 
 	"github.com/benhoyt/goawk/internal/ast"
 	"github.com/benhoyt/goawk/lexer"
@@ -415,6 +416,9 @@ func (c *compiler) stmt(stmt ast.Stmt) {
 	case *ast.NextStmt:
 		c.add(Next)
 
+	case *ast.NextfileStmt:
+		c.add(Nextfile)
+
 	case *ast.ExitStmt:
 		if s.Status != nil {
 			c.expr(s.Status)
@@ -880,6 +884,9 @@ func (c *compiler) expr(expr ast.Expr) {
 			c.add(Getline, redirect())
 		}
 
+	case *ast.GroupingExpr:
+		c.expr(e.Expr)
+
 	default:
 		// Should never happen
 		panic(fmt.Sprintf("unexpected expr type: %T", expr))
@@ -997,6 +1004,13 @@ func (c *compiler) binaryOp(op lexer.Token) {
 // Generate an array index, handling multi-indexes properly.
 func (c *compiler) index(index []ast.Expr) {
 	for _, expr := range index {
+		if e, ok := expr.(*ast.NumExpr); ok && e.Value == float64(int(e.Value)) {
+			// If index expression is integer constant, optimize to string "n"
+			// to avoid toString() at runtime.
+			s := strconv.Itoa(int(e.Value))
+			c.expr(&ast.StrExpr{Value: s})
+			continue
+		}
 		c.expr(expr)
 	}
 	if len(index) > 1 {
