@@ -636,6 +636,7 @@ func nextRune(b []byte) rune {
 func (p *interp) setFile(filename string) {
 	p.filename = numStr(filename)
 	p.fileLineNum = 0
+	p.hadFiles = true
 }
 
 // Setup for a new input line (but don't parse it into fields till we
@@ -721,8 +722,7 @@ func (p *interp) nextLine() (string, error) {
 				// Moved past number of ARGV args and haven't seen
 				// any files yet, use stdin
 				p.input = p.stdin
-				p.setFile("")
-				p.hadFiles = true
+				p.setFile("-")
 			} else {
 				if p.filenameIndex >= p.argc {
 					// Done with ARGV args, all done with input
@@ -738,7 +738,10 @@ func (p *interp) nextLine() (string, error) {
 				p.filenameIndex++
 
 				// Is it actually a var=value assignment?
-				matches := varRegex.FindStringSubmatch(filename)
+				var matches []string
+				if !p.noArgVars {
+					matches = varRegex.FindStringSubmatch(filename)
+				}
 				if len(matches) >= 3 {
 					// Yep, set variable to value and keep going
 					name, val := matches[1], matches[2]
@@ -759,7 +762,7 @@ func (p *interp) nextLine() (string, error) {
 				} else if filename == "-" {
 					// ARGV arg is "-" meaning stdin
 					p.input = p.stdin
-					p.setFile("")
+					p.setFile("-")
 				} else {
 					// A regular file name, open it
 					if p.noFileReads {
@@ -771,7 +774,6 @@ func (p *interp) nextLine() (string, error) {
 					}
 					p.input = input
 					p.setFile(filename)
-					p.hadFiles = true
 				}
 			}
 			if p.inputBuffer == nil { // reuse buffer from last input file
