@@ -271,7 +271,7 @@ func (p *parser) simpleStmt() ast.Stmt {
 			p.expect(RBRACKET)
 		}
 		return &ast.DeleteStmt{ref, index, startPos, p.pos}
-	case IF, FOR, WHILE, DO, BREAK, CONTINUE, NEXT, EXIT, RETURN:
+	case IF, FOR, WHILE, DO, BREAK, CONTINUE, NEXT, NEXTFILE, EXIT, RETURN:
 		panic(p.errorf("expected print/printf, delete, or expression"))
 	default:
 		return &ast.ExprStmt{p.expr(), startPos, p.pos}
@@ -394,6 +394,12 @@ func (p *parser) stmt() ast.Stmt {
 		}
 		p.next()
 		s = &ast.NextStmt{startPos, p.pos}
+	case NEXTFILE:
+		if !p.inAction && p.funcName == "" {
+			panic(p.errorf("nextfile can't be inside BEGIN or END"))
+		}
+		p.next()
+		s = &ast.NextfileStmt{startPos, p.pos}
 	case EXIT:
 		p.next()
 		var status ast.Expr
@@ -530,37 +536,57 @@ func (p *parser) getLine() ast.Expr {
 // An lvalue is a variable name, an array[expr] index expression, or
 // an $expr field expression.
 func (p *parser) _assign(higher func() ast.Expr) ast.Expr {
+	leftPos := p.pos
 	expr := higher()
-	_, isNamedField := expr.(*ast.NamedFieldExpr)
-	if (isNamedField || ast.IsLValue(expr)) && p.matches(ASSIGN, ADD_ASSIGN, DIV_ASSIGN,
-		MOD_ASSIGN, MUL_ASSIGN, POW_ASSIGN, SUB_ASSIGN) {
+	if p.matches(ASSIGN, ADD_ASSIGN, DIV_ASSIGN, MOD_ASSIGN, MUL_ASSIGN, POW_ASSIGN, SUB_ASSIGN) {
+		_, isNamedField := expr.(*ast.NamedFieldExpr)
 		if isNamedField {
 			panic(p.errorf("assigning @ expression not supported"))
 		}
 		op := p.tok
 		p.next()
 		right := p._assign(higher)
-		switch op {
-		case ASSIGN:
-			return &ast.AssignExpr{expr, right}
-		case ADD_ASSIGN:
-			op = ADD
-		case DIV_ASSIGN:
-			op = DIV
-		case MOD_ASSIGN:
-			op = MOD
-		case MUL_ASSIGN:
-			op = MUL
-		case POW_ASSIGN:
-			op = POW
-		case SUB_ASSIGN:
-			op = SUB
+		if !ast.IsLValue(expr) {
+			// Partial backtracking to allow expressions like "1 && x=1",
+			// which isn't really valid, as assignments are lower-precedence
+			// than binary operators, but onetrueawk, Gawk, and mawk all
+			// support this for logical, match and comparison operators. See
+			// issue #166.
+			binary, isBinary := expr.(*ast.BinaryExpr)
+			if isBinary && ast.IsLValue(binary.Right) {
+				switch binary.Op {
+				case AND, OR, MATCH, NOT_MATCH, EQUALS, NOT_EQUALS, LESS, LTE, GTE, GREATER:
+					assign := makeAssign(binary.Right, op, right)
+					return &ast.BinaryExpr{binary.Left, binary.Op, assign}
+				}
+			}
+			panic(ast.PosErrorf(leftPos, "expected lvalue before %s", op))
 		}
-		return &ast.AugAssignExpr{expr, op, right}
+		return makeAssign(expr, op, right)
 	}
 	return expr
 }
 
+func makeAssign(left ast.Expr, op Token, right ast.Expr) ast.Expr {
+	switch op {
+	case ASSIGN:
+		return &ast.AssignExpr{left, right}
+	case ADD_ASSIGN:
+		op = ADD
+	case DIV_ASSIGN:
+		op = DIV
+	case MOD_ASSIGN:
+		op = MOD
+	case MUL_ASSIGN:
+		op = MUL
+	case POW_ASSIGN:
+		op = POW
+	case SUB_ASSIGN:
+		op = SUB
+	}
+	return &ast.AugAssignExpr{left, op, right}
+}
+
 // Parse a ?: conditional expression:
 //
 //	or [QUESTION NEWLINE* cond COLON NEWLINE* cond]
@@ -664,7 +690,7 @@ func (p *parser) mul() ast.Expr {
 
 func (p *parser) pow() ast.Expr {
 	// Note that pow (expr ^ expr) is right-associative
-	expr := p.preIncr()
+	expr := p.postIncr()
 	if p.tok == POW {
 		p.next()
 		right := p.pow()
@@ -673,20 +699,6 @@ func (p *parser) pow() ast.Expr {
 	return expr
 }
 
-func (p *parser) preIncr() ast.Expr {
-	if p.tok == INCR || p.tok == DECR {
-		op := p.tok
-		p.next()
-		exprPos := p.pos
-		expr := p.preIncr()
-		if !ast.IsLValue(expr) {
-			panic(ast.PosErrorf(exprPos, "expected lvalue after ++ or --"))
-		}
-		return &ast.IncrExpr{expr, op, true}
-	}
-	return p.postIncr()
-}
-
 func (p *parser) postIncr() ast.Expr {
 	expr := p.primary()
 	if (p.tok == INCR || p.tok == DECR) && ast.IsLValue(expr) {
@@ -724,6 +736,15 @@ func (p *parser) primary() ast.Expr {
 		op := p.tok
 		p.next()
 		return &ast.UnaryExpr{op, p.pow()}
+	case INCR, DECR:
+		op := p.tok
+		p.next()
+		exprPos := p.pos
+		expr := p.optionalLValue()
+		if expr == nil {
+			panic(ast.PosErrorf(exprPos, "expected lvalue after %s", op))
+		}
+		return &ast.IncrExpr{expr, op, true}
 	case NAME:
 		name := p.val
 		namePos := p.pos
@@ -753,7 +774,7 @@ func (p *parser) primary() ast.Expr {
 			panic(p.errorf("expected expression, not %s", p.tok))
 		case 1:
 			p.expect(RPAREN)
-			return exprs[0]
+			return &ast.GroupingExpr{exprs[0]}
 		default:
 			// Multi-dimensional array "in" requires parens around index
 			p.expect(RPAREN)
